<?php
/**
 * Copyright © 2019 Cart2cart
 * created by Cart2cart (contact@shopping-cart-migration.com)
 */
namespace Cart2Cart\MigrationToMagento\Controller\Bridge;

use Magento\Framework\App\Action\Context;

class Install extends \Magento\Framework\App\Action\Action
{
    const MODULE_DEFAULT = 5060;
    const MODULE_BRIDGE_ALREADY_INSTALLED = 5061;
    const MODULE_BRIDGE_SUCCESSFULLY_INSTALLED = 5062;

    const MODULE_ERROR_DEFAULT = 'defaultError';
    const MODULE_ERROR_TOKEN = 5068;
    const MODULE_ERROR_ROOT_DIR_PERMISSION = 5069;
    const MODULE_ERROR_EXTRACTING_ARCHIVE = 5070;
    const MODULE_ERROR_INSTALLED_PERMISSION = 5071;
    const MODULE_ERROR_PERMISSION = 5072;

    public $resultPageFactory;
    public $rootPath;
    public $resultJsonFactory;
    public $file;

    const API_PATH_FOR_BRIDGE = 'https://app.shopping-cart-migration.com/api.bridge.download/token/';

    public function __construct(
        Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\App\Filesystem\DirectoryList $directoryList,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Framework\Filesystem\Driver\File $file,
        \Magento\Framework\App\DeploymentConfig $deploymentConfig
    ) {
        $this->resultPageFactory = $resultPageFactory;
        $this->rootPath = $directoryList->getRoot();

        if ($deploymentConfig->get('directories/document_root_is_pub')) {
          $this->rootPath = $directoryList->getPath('pub');
        }

        $this->resultJsonFactory = $resultJsonFactory;
        $this->file = $file;
        parent::__construct($context);
    }

    public function execute()
    {
        $token = $this->getRequest()->getParam('token', '');
        $mode = $this->getRequest()->getParam('mode', '');

        if ($mode === 'install') {
            return $this->installBridge($token);
        }

        $this->_redirect('');
    }

    protected function installBridge($token)
    {
        $response = new \stdClass();
        $rootPath = $this->rootPath;

        $this->copyBridge($rootPath, $token, $response);

        if (!isset($response->code)) {
            $response->code = $response->isError ? self::MODULE_ERROR_DEFAULT : self::MODULE_DEFAULT;
        }

        if (!isset($response->message)) {
            $response->message = '';
        }

        return $this->_viewResponse($response);
    }

    /**
     * @param string    $rootPath
     * @param string    $token
     * @param \stdClass $response
     *
     * @return void
     */
    protected function copyBridge($rootPath, $token, \stdClass $response)
    {
        try {
            if (!$token) {
                $response->isError = true;
                $response->code = self::MODULE_ERROR_TOKEN;
                $response->message = 'Token can\'t be blank.';
                return;
            }

            if ($this->isBridgeInstalled()) {
                $response->isError = false;
                $response->code = self::MODULE_BRIDGE_ALREADY_INSTALLED;
                $response->message = 'Bridge already installed.';
                return;
            }

            if (!$this->_checkMagentoRootDirPermission()) {
                $response->isError = true;
                $response->code = self::MODULE_ERROR_ROOT_DIR_PERMISSION;
                $response->message = 'Bad permission for Magento root folder.';
                return;
            }

            $arrContextOptions = array(
              "ssl" => array(
                "verify_peer" => false,
                "verify_peer_name" => false,
              ),
            );
            $zippedBridge = file_get_contents(self::API_PATH_FOR_BRIDGE . $token, false, stream_context_create($arrContextOptions));

            if (function_exists('file_put_contents')) {
                file_put_contents($rootPath . DIRECTORY_SEPARATOR . 'bridge.zip', $zippedBridge);
            }

            $zip = new \ZipArchive();
            if ($zip->open($rootPath . DIRECTORY_SEPARATOR . 'bridge.zip')) {
                $zip->extractTo($rootPath . DIRECTORY_SEPARATOR);
                $zip->close();

                $fileName = 'bridge.zip';
                if ($this->file->isExists($rootPath . DIRECTORY_SEPARATOR . $fileName)) {
                    $this->file->deleteFile($rootPath . DIRECTORY_SEPARATOR . $fileName);
                }

                $response->isError = false;
                $response->code = self::MODULE_BRIDGE_SUCCESSFULLY_INSTALLED;

                $this->_checkBridgeDirPermission();
                $this->_checkBridgeFilesPermission();
            } else {
                $response->isError = true;
                $response->code = self::MODULE_ERROR_EXTRACTING_ARCHIVE;
                $response->message = 'Error extracting archive.';
            }
        } catch (\Error $e) {
            $response->isError = true;
            $response->message = $e->getMessage();
        } catch (\Exception $e) {
            $response->isError = true;

            if (strpos($e->getMessage(), 'ZipArchive') !== false) {
                $response->code = self::MODULE_ERROR_INSTALLED_PERMISSION;
                $response->message = 'Permission denied on ZipArchive save.';
            } elseif (strpos($e->getMessage(), 'Permission') !== false) {
                $response->code = self::MODULE_ERROR_PERMISSION;
                $response->message = 'Permission problem.';
            } else {
                $response->message = $e->getMessage();
            }
        }
    }

    protected function isBridgeInstalled()
    {
        $this->_checkBridgeDirPermission();
        $this->_checkBridgeFilePermission();
        return (file_exists($this->rootPath . DIRECTORY_SEPARATOR .  'bridge2cart' . DIRECTORY_SEPARATOR . 'bridge.php'));
    }

    /**
     * @return bool
     */
    protected function _checkBridgeDirPermission()
    {
        return $this->_checkDirPermission($this->rootPath . DIRECTORY_SEPARATOR .  'bridge2cart' . DIRECTORY_SEPARATOR);
    }

    /**
     * @return bool
     */
    protected function _checkMagentoRootDirPermission()
    {
        return $this->_checkDirPermission($this->rootPath);
    }

    /**
     * @return bool
     */
    protected function _checkDirPermission($path)
    {
        if (!is_writable($path)) {
            $e = error_reporting();
            error_reporting(0);
            chmod($path, 0777);
            error_reporting($e);
        }

        return is_writable($path);
    }

    /**
     * @return bool
     */
    protected function _checkBridgeFilesPermission()
    {
        $check = true;

        $files = [
            $this->rootPath . DIRECTORY_SEPARATOR .  'bridge2cart' . DIRECTORY_SEPARATOR . 'bridge.php',
            $this->rootPath . DIRECTORY_SEPARATOR .  'bridge2cart' . DIRECTORY_SEPARATOR . 'config.php',
            $this->rootPath . DIRECTORY_SEPARATOR .  'bridge2cart' . DIRECTORY_SEPARATOR . 'index.php',
        ];

        foreach ($files as $file) {
            $check and $check = $this->_checkBridgeFilePermission($file);
        }

        return $check;
    }

    /**
     * @return bool
     */
    protected function _checkBridgeFilePermission($pathToFile = null)
    {
        if ($pathToFile === null) {
            $pathToFile = $this->rootPath . DIRECTORY_SEPARATOR .  'bridge2cart' . DIRECTORY_SEPARATOR . 'bridge.php';
        }

        if (!is_writable($pathToFile)) {
            $e = error_reporting();
            error_reporting(0);
            chmod($pathToFile, 0666);
            error_reporting($e);
        }

        return is_writable($pathToFile);
    }


    /**
     * @param \stdClass $response response
     */
    protected function _viewResponse(\stdClass $response)
    {
        header('Access-Control-Allow-Origin: *');
        $result = $this->resultJsonFactory->create();
        return $result->setData($response);
    }
}
