<?php

namespace Cart2Cart\MigrationToMagento\Helper\Encryption;

class Zencart implements EncryptorInterface
{
    protected static $instance = null;

    public function password_hash($plain, $algo = null, array $options = array())
    {
        $phpVersion = PHP_VERSION;

        if (version_compare($phpVersion, '5.5.0', '<')) {

            if (!function_exists('crypt')) {
                return false;
            }
            if (!is_string($plain)) {
                return false;
            }
            if (!is_int($algo)) {
                return false;
            }
            switch ($algo) {
                case PASSWORD_BCRYPT:
                    // Note that this is a C constant, but not exposed to PHP, so we don't define it here.
                    $cost = 10;
                    if (isset($options['cost'])) {
                        $cost = $options['cost'];
                        if ($cost < 4 || $cost > 31) {
                            trigger_error(sprintf("password_hash(): Invalid bcrypt cost parameter specified: %d", $cost), E_USER_WARNING);
                            return null;
                        }
                    }
                    // The length of salt to generate
                    $raw_salt_len = 16;
                    // The length required in the final serialization
                    $required_salt_len = 22;
                    $hash_format = sprintf("$2y$%02d$", $cost);
                break;
                default:
                    trigger_error(sprintf("password_hash(): Unknown password hashing algorithm: %s", $algo), E_USER_WARNING);
                    return null;
            }
            if (isset($options['salt'])) {
                switch (gettype($options['salt'])) {
                    case 'NULL':
                    case 'boolean':
                    case 'integer':
                    case 'double':
                    case 'string':
                        $salt = (string) $options['salt'];
                    break;
                    case 'object':
                        if (method_exists($options['salt'], '__tostring')) {
                            $salt = (string) $options['salt'];
                            break;
                        }
                    case 'array':
                    case 'resource':
                    default:
                        trigger_error('password_hash(): Non-string salt parameter supplied', E_USER_WARNING);
                        return null;
                }
                if (strlen($salt) < $required_salt_len) {
                    trigger_error(sprintf("password_hash(): Provided salt is too short: %d expecting %d", strlen($salt), $required_salt_len), E_USER_WARNING);
                    return null;
                } elseif (0 == preg_match('#^[a-zA-Z0-9./]+$#D', $salt)) {
                    $salt = str_replace('+', '.', base64_encode($salt));
                }
            } else {
                $buffer = '';
                $buffer_valid = false;
                if (function_exists('mcrypt_create_iv') && defined('MCRYPT_DEV_URANDOM') && !defined('PHALANGER')) {
                  $buffer = call_user_func('mcrypt_create_iv', $raw_salt_len, 1);
                  if ($buffer) {
                    $buffer_valid = true;
                  }
                }
                if (!$buffer_valid && function_exists('openssl_random_pseudo_bytes')) {
                    $buffer = openssl_random_pseudo_bytes($raw_salt_len);
                    if ($buffer) {
                        $buffer_valid = true;
                    }
                }
                if (!$buffer_valid && is_readable('/dev/urandom')) {
                    $f = fopen('/dev/urandom', 'r');
                    $read = strlen($buffer);
                    while ($read < $raw_salt_len) {
                        $buffer .= fread($f, $raw_salt_len - $read);
                        $read = strlen($buffer);
                    }
                    fclose($f);
                    if ($read >= $raw_salt_len) {
                        $buffer_valid = true;
                    }
                }
                if (!$buffer_valid || strlen($buffer) < $raw_salt_len) {
                    $bl = strlen($buffer);
                    for ($i = 0; $i < $raw_salt_len; $i++) {
                        if ($i < $bl) {
                            $buffer[$i] = $buffer[$i] ^ chr(rand(0, 255));
                        } else {
                            $buffer .= chr(rand(0, 255));
                        }
                    }
                }
                $salt = str_replace('+', '.', base64_encode($buffer));
            }
            $salt = substr($salt, 0, $required_salt_len);

            $hash = $hash_format . $salt;

            $ret = crypt($plain, $hash);

            if (!is_string($ret) || strlen($ret) <= 13) {
                return false;
            }

            return $ret;
        } else {
          return password_hash($plain, PASSWORD_DEFAULT);
        }
    }

    public function password_verify($password, $hash)
    {
        $phpVersion = PHP_VERSION;

        if (version_compare($phpVersion, '5.5.0', '<')) {

            if (!function_exists('crypt')) {
                trigger_error("Crypt must be loaded for password_verify to function", E_USER_WARNING);
                return false;
            }

            $ret = crypt($password, $hash);
            if (!is_string($ret) || strlen($ret) != strlen($hash) || strlen($ret) <= 13) {
                return false;
            }

            $status = 0;
            for ($i = 0; $i < strlen($ret); $i++) {
                $status |= (ord($ret[$i]) ^ ord($hash[$i]));
            }

            return $status === 0;
        } else {
          return password_verify($password, $hash);
        }
    }

    public function password_needs_rehash($hash, $algo = null, array $options = array())
    {
        $phpVersion = PHP_VERSION;

        if (version_compare($phpVersion, '5.5.0', '<')) {
            $info = $this->password_get_info($hash);
            if ($info['algo'] != $algo) {
                return true;
            }

            switch ($algo) {
                case PASSWORD_BCRYPT:
                    $cost = isset($options['cost']) ? $options['cost'] : 10;
                    if ($cost != $info['options']['cost']) {
                        return true;
                    }
                break;
            }

            return false;
        } else {
          return password_needs_rehash($hash, $algo, $options);
        }
    }

    public function password_get_info($hash) {
        $return = array(
            'algo' => 0,
            'algoName' => 'unknown',
            'options' => array(),
        );
        if (substr($hash, 0, 4) == '$2y$' && strlen($hash) == 60) {
            $return['algo'] = PASSWORD_BCRYPT;
            $return['algoName'] = 'bcrypt';
            list($cost) = sscanf($hash, "$2y$%d$");
            $return['options']['cost'] = $cost;
        }
        return $return;
    }

    public function validatePassword($hash, $password, $salt)
    {
        $type = $this->detectPasswordType($hash . ':'. $salt);
        if ($type != 'unknown') {
            $method = 'validatePassword' . ucfirst($type);
            return $this->{$method}($password, $hash . ':'. $salt);
        }

        $result = $this->password_verify($password, $hash);
        return $result;
    }

    public function detectPasswordType($encryptedPassword)
    {
        $type = 'unknown';
        $tmp = explode(':', $encryptedPassword);
        if (count($tmp) == 2) {
            if (strlen($tmp [1]) > 2) {
                $type = 'compatSha256';
            } elseif (strlen($tmp [1]) == 2) {
                $type = 'oldMd5';
            }
        }
        return $type;
    }

    public function validatePasswordOldMd5($plain, $encrypted)
    {
        if ($this->zen_not_null($plain) && $this->zen_not_null($encrypted)) {
            $stack = explode(':', $encrypted);
            if (sizeof($stack) != 2)
                return false;
            if (hash('md5', $stack [1] . $plain) == $stack [0]) {
                return true;
            }
        }
        return false;
    }

    public function validatePasswordCompatSha256($plain, $encrypted)
    {
        if ($this->zen_not_null($plain) && $this->zen_not_null($encrypted)) {
            $stack = explode(':', $encrypted);
            if (sizeof($stack) != 2)
                return false;
            if (hash('sha256', $stack [1] . $plain) == $stack [0]) {
                return true;
            }
        }
        return false;
    }

    public function hashPassword($plain)
    {
        $hash = $this->password_hash($plain, PASSWORD_DEFAULT);

        return $hash;
    }

    protected function zen_not_null($value)
    {
        if (is_array($value)) {
            if (sizeof($value) > 0) {
                return true;
            } else {
                return false;
            }
        } else {
            if (($value != '') && (strtolower($value) != 'null') && (strlen(trim($value)) > 0)) {
                return true;
            } else {
                return false;
            }
        }
    }
}
